@@ -6,7 +6,7 @@
 #ifndef _IP_VS_H
 #define _IP_VS_H
 
-#define IP_VS_VERSION_CODE            0x000902
+#define IP_VS_VERSION_CODE            0x000903
 #define NVERSION(version)                       \
 	(version >> 16) & 0xFF,                 \
 	(version >> 8) & 0xFF,                  \
@@ -42,36 +42,6 @@
 #define IP_VS_STATE_MASTER  1           /* started as master */
 #define IP_VS_STATE_BACKUP  2           /* started as backup */
 
-#define IP_VS_SCHEDNAME_MAXLEN         16
-#define IP_VS_IFNAME_MAXLEN            16
-
-struct ip_vs_rule_user {
-	/* global options */
-	int             tcp_timeout;    /* timeout values */
-	int             tcp_fin_timeout;
-	int             udp_timeout;
-	int             state;          /* sync daemon state */
-	char            mcast_ifn[IP_VS_IFNAME_MAXLEN];
-					/* multicast interface name */
-
-	/* virtual service options */
-	u_int16_t	protocol;
-	u_int32_t	vaddr;          /* virtual address */
-	u_int16_t	vport;
-	u_int32_t       vfwmark;        /* firwall mark of virtual */
-	char            sched_name[IP_VS_SCHEDNAME_MAXLEN];
-	unsigned	vs_flags;       /* virtual service flags */
-	unsigned        timeout;        /* persistent timeout in ticks */
-	u_int32_t	netmask;        /* persistent netmask */
-
-	/* destination specific options */
-	u_int32_t	daddr;          /* real destination address */
-	u_int16_t	dport;
-	unsigned        conn_flags;     /* destination flags */
-	int             weight;         /* destination weight */
-};
-
-
 /*
  *      IPVS socket options
  */
@@ -93,7 +63,9 @@ struct ip_vs_rule_user {
 #define IP_VS_SO_SET_MAX	IP_VS_SO_SET_STOPDAEMON
 
 #define IP_VS_SO_GET_VERSION	IP_VS_BASE_CTL
-#define IP_VS_SO_GET_MAX	IP_VS_SO_GET_VERSION
+#define IP_VS_SO_GET_SERVICE	(IP_VS_BASE_CTL+1)
+#define IP_VS_SO_GET_DESTS	(IP_VS_BASE_CTL+2)
+#define IP_VS_SO_GET_MAX	IP_VS_SO_GET_DESTS
 
 
 /*
@@ -113,7 +85,81 @@ struct ip_vs_rule_user {
 #define IP_VS_CONN_F_SEQ_MASK         0x0600    /* in/out sequence mask */
 #define IP_VS_CONN_F_NO_CPORT         0x0800    /* no client port set yet */
 
-#define NFMARK_IPVS_PROPERTY	      -0x100
+/* Move it to better place one day, for now keep it unique */
+#define NFC_IPVS_PROPERTY	0x10000
+
+#define IP_VS_SCHEDNAME_MAXLEN         16
+#define IP_VS_IFNAME_MAXLEN            16
+
+struct ip_vs_rule_user {
+	/* global options */
+	int             tcp_timeout;    /* timeout values */
+	int             tcp_fin_timeout;
+	int             udp_timeout;
+	int             state;          /* sync daemon state */
+	char            mcast_ifn[IP_VS_IFNAME_MAXLEN];
+					/* multicast interface name */
+
+	/* virtual service options */
+	u_int16_t	protocol;
+	u_int32_t	vaddr;          /* virtual address */
+	u_int16_t	vport;
+	u_int32_t       vfwmark;        /* firwall mark of virtual service*/
+	char            sched_name[IP_VS_SCHEDNAME_MAXLEN];
+	unsigned	vs_flags;       /* virtual service flags */
+	unsigned        timeout;        /* persistent timeout in ticks */
+	u_int32_t	netmask;        /* persistent netmask */
+
+	/* destination specific options */
+	u_int32_t	daddr;          /* destination address */
+	u_int16_t	dport;
+	unsigned        conn_flags;     /* destination flags */
+	int             weight;         /* destination weight */
+};
+
+
+/* The argument to IP_VS_SO_GET_SERVICE */
+struct ip_vs_get_service {
+	/* which service: user fills this in */
+	u_int16_t	protocol;
+	u_int32_t	addr;           /* virtual address */
+	u_int16_t	port;
+	u_int32_t       fwmark;         /* firwall mark of virtual service */
+
+	/* service options */
+	char            sched_name[IP_VS_SCHEDNAME_MAXLEN];
+	unsigned	flags;          /* virtual service flags */
+	unsigned        timeout;        /* persistent timeout in ticks */
+	u_int32_t	netmask;        /* persistent netmask */
+
+	/* number of real servers */
+	unsigned int    num_dests;
+};
+
+struct ip_vs_dest_entry {
+	u_int32_t	addr;           /* destination address */
+	u_int16_t	port;
+	unsigned	flags;		/* destination flags */
+	int		weight;         /* destination weight */
+	u_int16_t	activeconns;	/* active connections */
+	u_int16_t	inactconns;	/* inactive connections */
+};
+
+/* The argument to IP_VS_SO_GET_DESTS */
+struct ip_vs_get_dests {
+	/* which service: user fills this in */
+	u_int16_t	protocol;
+	u_int32_t	addr;           /* virtual address */
+	u_int16_t	port;
+	u_int32_t       fwmark;         /* firwall mark of virtual service */
+
+	/* number of real servers */
+	unsigned int    num_dests;
+
+	/* the real servers */
+	struct ip_vs_dest_entry entrytable[0];
+};
+
 
 #ifdef __KERNEL__
 
@@ -130,27 +176,27 @@ struct ip_vs_rule_user {
 
 #ifdef CONFIG_IP_VS_DEBUG
 extern int ip_vs_get_debug_level(void);
-#define IP_VS_DBG(level, msg...)                        \
-    do {                                                \
-	    if (level <= ip_vs_get_debug_level())       \
-		    printk(KERN_DEBUG "IPVS: " ## msg); \
+#define IP_VS_DBG(level, msg...)			\
+    do {						\
+	    if (level <= ip_vs_get_debug_level())	\
+		    printk(KERN_DEBUG "IPVS: " msg);	\
     } while (0)
 #else	/* NO DEBUGGING at ALL */
 #define IP_VS_DBG(level, msg...)  do {} while (0)
 #endif
 
 #define IP_VS_BUG() BUG()
-#define IP_VS_ERR(msg...) printk(KERN_ERR "IPVS: " ## msg )
-#define IP_VS_INFO(msg...) printk(KERN_INFO "IPVS: " ## msg )
+#define IP_VS_ERR(msg...) printk(KERN_ERR "IPVS: " msg)
+#define IP_VS_INFO(msg...) printk(KERN_INFO "IPVS: " msg)
 #define IP_VS_WARNING(msg...) \
-	printk(KERN_WARNING "IPVS: " ## msg)
+	printk(KERN_WARNING "IPVS: " msg)
 
 #ifdef CONFIG_IP_VS_DEBUG
-#define EnterFunction(level)                                            \
-    do {                                                                \
-	    if (level <= ip_vs_get_debug_level())                       \
-		    printk(KERN_DEBUG "Enter: %s, %s line %i\n",        \
-			   __FUNCTION__, __FILE__, __LINE__);           \
+#define EnterFunction(level)						\
+    do {								\
+	    if (level <= ip_vs_get_debug_level())			\
+		    printk(KERN_DEBUG "Enter: %s, %s line %i\n",	\
+			   __FUNCTION__, __FILE__, __LINE__);		\
     } while (0)
 #define LeaveFunction(level)                                            \
     do {                                                                \
@@ -328,7 +374,9 @@ struct ip_vs_service {
 	unsigned		flags;	  /* service status flags */
 	unsigned		timeout;  /* persistent timeout in ticks */
 	__u32			netmask;  /* grouping granularity */
+
 	struct list_head	destinations;  /* real server d-linked list */
+	__u32			num_dests;     /* number of servers */
 	struct ip_vs_stats      stats;         /* statistics for the service */
 
 	/* for scheduling */
@@ -466,6 +514,8 @@ extern struct ip_vs_conn *
 ip_vs_conn_new(int proto, __u32 maddr, __u16 mport, __u32 saddr,
 	       __u16 sport, __u32 daddr, __u16 dport, unsigned flags,
 	       struct ip_vs_dest *dest);
+#define ip_vs_conn_expire_now(cp)  cp->timer.function((unsigned long)cp)
+
 extern const char * ip_vs_state_name(int state);
 extern int ip_vs_set_state(struct ip_vs_conn *cp, int state_off,
 			   struct iphdr *iph, void *tp);
