@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.18 2001/06/20 12:50:52 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.19 2001/08/27 02:40:54 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -632,7 +632,7 @@ static int ip_vs_bypass_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 #ifdef CONFIG_NETFILTER_DEBUG
 	skb->nf_debug = 1 << NF_IP_LOCAL_OUT;
 #endif /* CONFIG_NETFILTER_DEBUG */
-	skb->nfmark = NFMARK_IPVS_PROPERTY;
+	skb->nfcache |= NFC_IPVS_PROPERTY;
 	ip_send(skb);
 
 	LeaveFunction(10);
@@ -683,7 +683,7 @@ static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 		case CHECKSUM_NONE:
 			csum = csum_partial(h.raw + doff, size - doff, 0);
 			csum_ok++;
-			skb->csum = csum_partial(h.raw , doff, csum);
+			skb->csum = csum_partial(h.raw, doff, csum);
 
 		case CHECKSUM_HW:
 			if (csum_tcpudp_magic(iph->saddr, iph->daddr,
@@ -794,7 +794,7 @@ static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 #ifdef CONFIG_NETFILTER_DEBUG
 	skb->nf_debug = 1 << NF_IP_LOCAL_OUT;
 #endif /* CONFIG_NETFILTER_DEBUG */
-	skb->nfmark = NFMARK_IPVS_PROPERTY;
+	skb->nfcache |= NFC_IPVS_PROPERTY;
 	ip_send(skb);
 
 	LeaveFunction(10);
@@ -917,7 +917,7 @@ static int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 #ifdef CONFIG_NETFILTER_DEBUG
 	skb->nf_debug = 1 << NF_IP_LOCAL_OUT;
 #endif /* CONFIG_NETFILTER_DEBUG */
-	skb->nfmark = NFMARK_IPVS_PROPERTY;
+	skb->nfcache |= NFC_IPVS_PROPERTY;
 	ip_send(skb);
 
 	LeaveFunction(10);
@@ -962,7 +962,7 @@ static int ip_vs_dr_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 #ifdef CONFIG_NETFILTER_DEBUG
 	skb->nf_debug = 1 << NF_IP_LOCAL_OUT;
 #endif /* CONFIG_NETFILTER_DEBUG */
-	skb->nfmark = NFMARK_IPVS_PROPERTY;
+	skb->nfcache |= NFC_IPVS_PROPERTY;
 	ip_send(skb);
 
 #if 0000
@@ -1369,7 +1369,6 @@ void ip_vs_random_dropentry(void)
 	struct ip_vs_conn *cp;
 	struct list_head *l,*e;
 	struct ip_vs_conn *ct;
-	void (*fn)(unsigned long);
 
 	/*
 	 * Randomly scan 1/32 of the whole table every second
@@ -1408,14 +1407,13 @@ void ip_vs_random_dropentry(void)
 			 */
 			ct_write_unlock(hash);
 			IP_VS_DBG(4, "Drop connection\n");
-			fn = (cp->timer).function;
 			if (del_sltimer(&cp->timer)) {
 				ct = cp->control;
-				fn((unsigned long)cp);
+				ip_vs_conn_expire_now(cp);
 				if (ct && !atomic_read(&ct->n_control)) {
 					IP_VS_DBG(4, "Drop conn template\n");
 					if(del_sltimer(&ct->timer))
-						fn((unsigned long)ct);
+						ip_vs_conn_expire_now(ct);
 				}
 			}
 			ct_write_lock(hash);
@@ -1434,7 +1432,6 @@ static void ip_vs_conn_flush(void)
 	struct ip_vs_conn *cp;
 	struct list_head *l,*e;
 	struct ip_vs_conn *ct;
-	void (*fn)(unsigned long);
 
  flush_again:
 	for (idx=0; idx<IP_VS_CONN_TAB_SIZE; idx++) {
@@ -1448,14 +1445,14 @@ static void ip_vs_conn_flush(void)
 			cp = list_entry(e, struct ip_vs_conn, c_list);
 			ct_write_unlock(idx);
 			IP_VS_DBG(4, "del connection\n");
-			fn = (cp->timer).function;
 			if (del_sltimer(&cp->timer)) {
 				ct = cp->control;
-				fn((unsigned long)cp);
+				ip_vs_conn_expire_now(cp);
+
 				if (ct && !atomic_read(&ct->n_control)) {
 					IP_VS_DBG(4, "del conn template\n");
 					if(del_sltimer(&ct->timer))
-						fn((unsigned long)ct);
+						ip_vs_conn_expire_now(ct);
 				}
 			}
 			ct_write_lock(idx);
