@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.20 2001/06/20 12:50:52 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.21 2001/08/27 02:40:54 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -552,7 +552,7 @@ static unsigned int ip_vs_post_routing(unsigned int hooknum,
 {
 	struct sk_buff  *skb = *skb_p;
 
-	if (skb->nfmark != NFMARK_IPVS_PROPERTY)
+	if (!(skb->nfcache & NFC_IPVS_PROPERTY))
 		return NF_ACCEPT;
 
 	/* The packet was sent from IPVS, exit this chain */
@@ -610,8 +610,7 @@ static int ip_vs_out_icmp(struct sk_buff **skb_p)
 	pptr = (__u16 *)&(((char *)ciph)[ciph->ihl*4]);
 
 	/* Ensure the checksum is correct */
-	if (ip_compute_csum((unsigned char *) icmph, len))
-	{
+	if (ip_compute_csum((unsigned char *) icmph, len)) {
 		/* Failed checksum! */
 		IP_VS_DBG(1, "forward ICMP: failed checksum from %d.%d.%d.%d!\n",
 			  NIPQUAD(iph->saddr));
@@ -659,7 +658,7 @@ static int ip_vs_out_icmp(struct sk_buff **skb_p)
 		  NIPQUAD(ciph->saddr), ntohs(pptr[0]),
 		  NIPQUAD(ciph->daddr), ntohs(pptr[1]));
 
-	skb->nfmark = NFMARK_IPVS_PROPERTY;
+	skb->nfcache |= NFC_IPVS_PROPERTY;
 
 	return NF_ACCEPT;
 }
@@ -687,7 +686,7 @@ static unsigned int ip_vs_out(unsigned int hooknum,
 
 	EnterFunction(11);
 
-	if (skb->nfmark == NFMARK_IPVS_PROPERTY)
+	if (skb->nfcache & NFC_IPVS_PROPERTY)
 		return NF_ACCEPT;
 
 	iph = skb->nh.iph;
@@ -744,14 +743,13 @@ static unsigned int ip_vs_out(unsigned int hooknum,
 	csum = 0;
 	csum_ok = 0;
 	if ((iph->protocol != IPPROTO_UDP) || (h.uh->check != 0)) {
-		switch (skb->ip_summed)
-		{
+		switch (skb->ip_summed) {
 		case CHECKSUM_NONE:
 			csum = csum_partial(h.raw + doff, size - doff, 0);
+			csum_ok++;
 			IP_VS_DBG(11, "O-pkt: %s I-datacsum=%d\n",
 				  ip_vs_proto_name(iph->protocol), csum);
-			skb->csum = csum_partial(h.raw , doff, csum);
-			csum_ok++;
+			skb->csum = csum_partial(h.raw, doff, csum);
 		case CHECKSUM_HW:
 			if (csum_tcpudp_magic(iph->saddr, iph->daddr, size,
 					      iph->protocol, skb->csum)) {
@@ -848,7 +846,7 @@ static unsigned int ip_vs_out(unsigned int hooknum,
 	ip_vs_set_state(cp, VS_STATE_OUTPUT, iph, h.portp);
 	ip_vs_conn_put(cp);
 
-	skb->nfmark = NFMARK_IPVS_PROPERTY;
+	skb->nfcache |= NFC_IPVS_PROPERTY;
 
 	LeaveFunction(11);
 	return NF_ACCEPT;
@@ -872,7 +870,7 @@ unsigned int check_for_ip_vs_out(struct sk_buff **skb_p,
 	} else {
 		/* send the packet immediately if it is already mangled
 		   by ip_vs_out */
-		if ((*skb_p)->nfmark == NFMARK_IPVS_PROPERTY) {
+		if ((*skb_p)->nfcache & NFC_IPVS_PROPERTY) {
 			(*okfn)(*skb_p);
 			return NF_STOLEN;
 		}
@@ -920,8 +918,7 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
 		return NF_ACCEPT;
 
 	/* Ensure the checksum is correct */
-	if (ip_compute_csum((unsigned char *) icmph, len))
-	{
+	if (ip_compute_csum((unsigned char *) icmph, len)) {
 		/* Failed checksum! */
 		IP_VS_ERR("incoming ICMP: failed checksum from %d.%d.%d.%d!\n",
 			  NIPQUAD(iph->saddr));
@@ -1098,11 +1095,8 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 		if (sysctl_ip_vs_expire_nodest_conn) {
 			/* expire the connection immediately */
 			__ip_vs_conn_put(cp);
-			if (del_sltimer(&cp->timer)) {
-				void (*fn)(unsigned long)
-					= (cp->timer).function;
-				fn((unsigned long)cp);
-			}
+			if (del_sltimer(&cp->timer))
+				ip_vs_conn_expire_now(cp);
 
 			return NF_DROP;
 		} else {
