@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.18 2001/06/20 12:50:52 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.19 2001/08/27 02:40:54 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -44,8 +44,8 @@
 #include "ip_vs.h"
 //#include <net/ip_vs.h>
 
-/* spinlock for do_ip_vs_set_ctl */
-static spinlock_t __ip_vs_mutex = SPIN_LOCK_UNLOCKED;
+/* semaphore for IPVS sockopts. And, [gs]etsockopt may sleep. */
+static DECLARE_MUTEX(__ip_vs_mutex);
 
 /* lock for service table */
 rwlock_t __ip_vs_svc_lock = RW_LOCK_UNLOCKED;
@@ -798,6 +798,7 @@ static int ip_vs_add_dest(struct ip_vs_service *svc,
 		while (atomic_read(&svc->usecnt) > 1) {};
 
 		list_add(&dest->n_list, &svc->destinations);
+		svc->num_dests++;
 
 		/* call the update_service function of its scheduler */
 		svc->scheduler->update_service(svc);
@@ -827,6 +828,7 @@ static int ip_vs_add_dest(struct ip_vs_service *svc,
 	while (atomic_read(&svc->usecnt) > 1) {};
 
 	list_add(&dest->n_list, &svc->destinations);
+	svc->num_dests++;
 
 	/* call the update_service function of its scheduler */
 	svc->scheduler->update_service(svc);
@@ -897,7 +899,7 @@ static void __ip_vs_del_dest(struct ip_vs_dest *dest)
 		/* simply decrease svc->refcnt here, let the caller check
 		   and release the service if nobody refers to it.
 		   Only user context can release destination and service,
-		   and only user context can update virtual service at a
+		   and only one user context can update virtual service at a
 		   time, so the operation here is OK */
 		atomic_dec(&dest->svc->refcnt);
 		kfree(dest);
@@ -924,6 +926,7 @@ static void __ip_vs_unlink_dest(struct ip_vs_service *svc,
 	 *  Remove it from the d-linked destination list.
 	 */
 	list_del(&dest->n_list);
+	svc->num_dests--;
 	if (svcupd) {
 		/*
 		 *  Call the update_service function of its scheduler
@@ -1761,8 +1764,7 @@ do_ip_vs_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
 	}
 
 	MOD_INC_USE_COUNT;
-
-	spin_lock(&__ip_vs_mutex);
+	down_interruptible(&__ip_vs_mutex);
 
 	if (cmd == IP_VS_SO_SET_FLUSH) {
 		/* Flush the virtual service */
@@ -1843,14 +1845,13 @@ do_ip_vs_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
 		break;
 	default:
 		ret = -EINVAL;
-		break;
 	}
 
 	if (svc)
 		ip_vs_service_put(svc);
 
  out_unlock:
-	spin_unlock(&__ip_vs_mutex);
+	up(&__ip_vs_mutex);
 	MOD_DEC_USE_COUNT;
  out_free:
 	kfree(urule);
@@ -1858,24 +1859,130 @@ do_ip_vs_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
 }
 
 
+static inline int
+__ip_vs_get_dest_entries(const struct ip_vs_get_dests *get,
+			 struct ip_vs_get_dests *uptr)
+{
+	struct ip_vs_service *svc;
+	int ret = 0;
+
+	down_interruptible(&__ip_vs_mutex);
+	if (get->fwmark)
+		svc = __ip_vs_svc_fwm_get(get->fwmark);
+	else
+		svc = __ip_vs_service_get(get->protocol,
+					  get->addr, get->port);
+	if (svc) {
+		int i = 0;
+		struct ip_vs_dest *dest;
+		struct list_head *l, *e;
+		struct ip_vs_dest_entry entry;
+
+		l = &svc->destinations;
+		for (e=l->next; e!=l; e=e->next) {
+			dest = list_entry(e, struct ip_vs_dest, n_list);
+			entry.addr = dest->addr;
+			entry.port = dest->port;
+			entry.flags = atomic_read(&dest->conn_flags);
+			entry.weight = atomic_read(&dest->weight);
+			entry.activeconns = atomic_read(&dest->activeconns);
+			entry.inactconns = atomic_read(&dest->inactconns);
+			if (copy_to_user(&uptr->entrytable[i],
+					 &entry, sizeof(entry))) {
+				ret = -EFAULT;
+				break;
+			}
+			if (++i >= get->num_dests)
+				break;
+		}
+		ip_vs_service_put(svc);
+	} else
+		ret = -EEXIST;
+	up(&__ip_vs_mutex);
+	return ret;
+}
+
+
 static int
 do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 {
-	static char buf[32];
+	int ret = 0;
 
-	if (cmd != IP_VS_SO_GET_VERSION)
-		return -EINVAL;
+	if (!capable(CAP_NET_ADMIN))
+		return -EPERM;
 
-	sprintf(buf, "IPVS %d.%d.%d", NVERSION(IP_VS_VERSION_CODE));
+	switch (cmd) {
+	case IP_VS_SO_GET_VERSION:
+	{
+		char buf[32];
 
-	if (*len < sizeof(buf)+1)
-		return -EINVAL;
+		sprintf(buf, "IPVS %d.%d.%d", NVERSION(IP_VS_VERSION_CODE));
+		if (*len < sizeof(buf)+1)
+			return -EINVAL;
+		if (copy_to_user(user, buf, sizeof(buf)+1) != 0)
+			return -EFAULT;
+		*len = sizeof(buf)+1;
+	}
+	break;
 
-	if (copy_to_user(user, buf, sizeof(buf)+1) != 0)
-		return -EFAULT;
+	case IP_VS_SO_GET_SERVICE:
+	{
+		struct ip_vs_get_service get;
+		struct ip_vs_service *svc;
 
-	*len = sizeof(buf)+1;
-	return 0;
+		if (*len != sizeof(get)) {
+			IP_VS_ERR("length: %u != %u\n", *len, sizeof(get));
+			return -EINVAL;
+		}
+		if (copy_from_user(&get, user, sizeof(get)))
+			return -EFAULT;
+
+		down_interruptible(&__ip_vs_mutex);
+		if (get.fwmark)
+			svc = __ip_vs_svc_fwm_get(get.fwmark);
+		else
+			svc = __ip_vs_service_get(get.protocol,
+						  get.addr, get.port);
+		if (svc) {
+			strcpy(get.sched_name, svc->scheduler->name);
+			get.flags = svc->flags;
+			get.timeout = svc->timeout;
+			get.netmask = svc->netmask;
+			get.num_dests = svc->num_dests;
+			if (copy_to_user(user, &get, *len) != 0)
+				ret = -EFAULT;
+			ip_vs_service_put(svc);
+		} else
+			ret = -EEXIST;
+		up(&__ip_vs_mutex);
+	}
+	break;
+
+	case IP_VS_SO_GET_DESTS:
+	{
+		struct ip_vs_get_dests get;
+
+		if (*len < sizeof(get)) {
+			IP_VS_ERR("length: %u < %u\n", *len, sizeof(get));
+			return -EINVAL;
+		}
+		if (copy_from_user(&get, user, sizeof(get)))
+			return -EFAULT;
+		if (*len != (sizeof(get) +
+			     sizeof(struct ip_vs_dest_entry)*get.num_dests)) {
+			IP_VS_ERR("length: %u != %u\n", *len,
+				  sizeof(get)+sizeof(struct ip_vs_dest_entry)*get.num_dests);
+			return -EINVAL;
+		}
+		ret = __ip_vs_get_dest_entries(&get, user);
+	}
+	break;
+
+	default:
+		ret = -EINVAL;
+	}
+
+	return ret;
 }
 
 
