@@ -1,7 +1,7 @@
 /*
  * IPVS:        Weighted Least-Connection Scheduling module
  *
- * Version:     $Id: ip_vs_wlc.c,v 1.7 2001/05/09 15:01:15 wensong Exp $
+ * Version:     $Id: ip_vs_wlc.c,v 1.8 2001/06/04 13:51:03 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -65,10 +65,6 @@ ip_vs_wlc_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 
 	IP_VS_DBG(6, "ip_vs_wlc_schedule(): Scheduling...\n");
 
-	l = &svc->destinations;
-	if (l == l->next)
-		return NULL;
-
 	/*
 	 * We think the overhead of processing active connections is fifty
 	 * times than that of inactive conncetions in average. (This fifty
@@ -87,6 +83,7 @@ ip_vs_wlc_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 	 * new connection.
 	 */
 
+	l = &svc->destinations;
 	for (e=l->next; e!=l; e=e->next) {
 		least = list_entry(e, struct ip_vs_dest, n_list);
 		if (atomic_read(&least->weight) > 0) {
